@@ -57,6 +57,13 @@
  **/
 
 
+typedef struct _ContentType {
+	char *type, *subtype;
+	gboolean exists;
+} ContentType;
+
+extern void _g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
+
 static void g_mime_parser_class_init (GMimeParserClass *klass);
 static void g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass);
 static void g_mime_parser_finalize (GObject *object);
@@ -64,15 +71,23 @@ static void g_mime_parser_finalize (GObject *object);
 static void parser_init (GMimeParser *parser, GMimeStream *stream);
 static void parser_close (GMimeParser *parser);
 
-static GMimeObject *parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
+static GMimeObject *parser_construct_leaf_part (GMimeParser *parser, ContentType *content_type,
 						int *found);
-static GMimeObject *parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
+static GMimeObject *parser_construct_multipart (GMimeParser *parser, ContentType *content_type,
 						int *found);
 
 static GObjectClass *parent_class = NULL;
 
-#define SCAN_BUF 4096		/* size of read buffer */
-#define SCAN_HEAD 128		/* headroom guaranteed to be before each read buffer */
+/* size of read buffer */
+#define SCAN_BUF 4096
+
+/* headroom guaranteed to be before each read buffer */
+#define SCAN_HEAD 128
+
+/* conservative growth sizes */
+#define HEADER_INIT_SIZE 128
+#define HEADER_RAW_INIT_SIZE 1024
+
 
 enum {
 	GMIME_PARSER_STATE_ERROR = -1,
@@ -112,8 +127,12 @@ struct _GMimeParserPrivate {
 	char *rawptr;
 	size_t rawleft;
 	
-	gint64 headers_start;
-	gint64 header_start;
+	/* current message headerblock offsets */
+	gint64 headers_begin;
+	gint64 headers_end;
+	
+	/* current header field offset */
+	gint64 header_offset;
 	
 	short int state;
 	
@@ -125,7 +144,6 @@ struct _GMimeParserPrivate {
 	unsigned short int persist_stream:1;
 	unsigned short int respect_content_length:1;
 	
-	GMimeContentType *content_type;
 	struct _header_raw *headers;
 	
 	struct _boundary_stack *bounds;
@@ -317,16 +335,18 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->from_offset = -1;
 	priv->from_line = g_byte_array_new ();
 	
-	priv->headerbuf = g_malloc (SCAN_HEAD);
+	priv->headerbuf = g_malloc (HEADER_INIT_SIZE);
+	priv->headerleft = HEADER_INIT_SIZE - 1;
 	priv->headerptr = priv->headerbuf;
-	priv->headerleft = SCAN_HEAD - 1;
 	
-	priv->rawbuf = g_malloc (SCAN_HEAD);
+	priv->rawbuf = g_malloc (HEADER_RAW_INIT_SIZE);
+	priv->rawleft = HEADER_RAW_INIT_SIZE - 1;
 	priv->rawptr = priv->rawbuf;
-	priv->rawleft = SCAN_HEAD - 1;
 	
-	priv->headers_start = -1;
-	priv->header_start = -1;
+	priv->headers_begin = -1;
+	priv->headers_end = -1;
+	
+	priv->header_offset = -1;
 	
 	priv->midline = FALSE;
 	priv->seekable = offset != -1;
@@ -366,11 +386,7 @@ parser_close (GMimeParser *parser)
 GMimeParser *
 g_mime_parser_new (void)
 {
-	GMimeParser *parser;
-	
-	parser = g_object_new (GMIME_TYPE_PARSER, NULL);
-	
-	return parser;
+	return g_object_newv (GMIME_TYPE_PARSER, 0, NULL);
 }
 
 
@@ -396,7 +412,7 @@ g_mime_parser_new_with_stream (GMimeStream *stream)
 
 /**
  * g_mime_parser_init_with_stream:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  * @stream: raw message or part stream
  *
  * Initializes @parser to use @stream.
@@ -428,7 +444,7 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
 
 /**
  * g_mime_parser_get_persist_stream:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets whether or not the underlying stream is persistent.
  *
@@ -446,7 +462,7 @@ g_mime_parser_get_persist_stream (GMimeParser *parser)
 
 /**
  * g_mime_parser_set_persist_stream:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  * @persist: persist attribute
  *
  * Sets whether or not the @parser's underlying stream is persistent.
@@ -473,7 +489,7 @@ g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
 
 /**
  * g_mime_parser_get_scan_from:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets whether or not @parser is set to scan mbox-style From-lines.
  *
@@ -491,7 +507,7 @@ g_mime_parser_get_scan_from (GMimeParser *parser)
 
 /**
  * g_mime_parser_set_scan_from:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  * @scan_from: %TRUE to scan From-lines or %FALSE otherwise
  *
  * Sets whether or not @parser should scan mbox-style From-lines.
@@ -507,7 +523,7 @@ g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
 
 /**
  * g_mime_parser_get_respect_content_length:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets whether or not @parser is set to use Content-Length for
  * determining the offset of the end of the message.
@@ -526,7 +542,7 @@ g_mime_parser_get_respect_content_length (GMimeParser *parser)
 
 /**
  * g_mime_parser_set_respect_content_length:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  * @respect_content_length: %TRUE if the parser should use Content-Length headers or %FALSE otherwise.
  *
  * Sets whether or not @parser should respect Content-Length headers
@@ -547,7 +563,7 @@ g_mime_parser_set_respect_content_length (GMimeParser *parser, gboolean respect_
 
 /**
  * g_mime_parser_set_header_regex:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  * @regex: regular expression
  * @header_cb: callback function
  * @user_data: user data
@@ -582,7 +598,7 @@ g_mime_parser_set_header_regex (GMimeParser *parser, const char *regex,
 
 
 static ssize_t
-parser_fill (GMimeParser *parser)
+parser_fill (GMimeParser *parser, size_t atleast)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	char *inbuf, *inptr, *inend;
@@ -596,6 +612,9 @@ parser_fill (GMimeParser *parser)
 	
 	g_assert (inptr <= inend);
 	
+	if (inlen > atleast)
+		return inlen;
+	
 	/* attempt to align 'inend' with realbuf + SCAN_HEAD */
 	if (inptr >= inbuf) {
 		inbuf -= inlen < SCAN_HEAD ? inlen : SCAN_HEAD;
@@ -616,12 +635,12 @@ parser_fill (GMimeParser *parser)
 	
 	priv->inptr = inptr;
 	priv->inend = inbuf;
-	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF - 1;
+	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF;
 	
-	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0)
+	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0) {
+		priv->offset += nread;
 		priv->inend += nread;
-	
-	priv->offset = g_mime_stream_tell (priv->stream);
+	}
 	
 	return priv->inend - priv->inptr;
 }
@@ -642,12 +661,12 @@ parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
 
 /**
  * g_mime_parser_tell:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets the current stream offset from the parser's internal stream.
  *
  * Returns: the current stream offset from the parser's internal stream
- * or -1 on error.
+ * or %-1 on error.
  **/
 gint64
 g_mime_parser_tell (GMimeParser *parser)
@@ -661,7 +680,7 @@ g_mime_parser_tell (GMimeParser *parser)
 
 /**
  * g_mime_parser_eos:
- * @parser: MIME parser
+ * @parser: a #GMimeParser context
  *
  * Tests the end-of-stream indicator for @parser's internal stream.
  *
@@ -695,7 +714,7 @@ parser_step_from (GMimeParser *parser)
 	
 	do {
 	refill:
-		if (parser_fill (parser) <= left) {
+		if (parser_fill (parser, MAX (SCAN_HEAD, left)) <= left) {
 			/* failed to find a From line; EOF reached */
 			priv->state = GMIME_PARSER_STATE_ERROR;
 			priv->inptr = priv->inend;
@@ -812,7 +831,7 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	if (*inptr != ':') {
 		/* ignore invalid headers */
 		w(g_warning ("Invalid header at %lld: '%s'",
-			     (long long) priv->header_start,
+			     (long long) priv->header_offset,
 			     priv->headerbuf));
 		
 		priv->headerleft += priv->headerptr - priv->headerbuf;
@@ -844,7 +863,7 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	
 	header->value = g_strndup (start, end - start);
 	
-	header->offset = priv->header_start;
+	header->offset = priv->header_offset;
 	
 	(*tail)->next = header;
 	*tail = header;
@@ -874,15 +893,17 @@ parser_step_headers (GMimeParser *parser)
 	raw_header_reset (priv);
 	header_raw_clear (&priv->headers);
 	hend = (struct _header_raw *) &priv->headers;
-	priv->headers_start = parser_offset (priv, NULL);
-	priv->header_start = parser_offset (priv, NULL);
+	priv->header_offset = parser_offset (priv, NULL);
+	
+	if (priv->headers_begin == -1)
+		priv->headers_begin = parser_offset (priv, NULL);
 	
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
 	do {
 	refill:
-		if (parser_fill (parser) <= left)
+		if (parser_fill (parser, MAX (SCAN_HEAD, left)) <= left)
 			break;
 		
 		inptr = priv->inptr;
@@ -898,7 +919,7 @@ parser_step_headers (GMimeParser *parser)
 			/* if we are scanning a new line, check for a folded header */
 			if (!priv->midline && continuation && (*inptr != ' ' && *inptr != '\t')) {
 				header_parse (parser, &hend);
-				priv->header_start = parser_offset (priv, inptr);
+				priv->header_offset = parser_offset (priv, inptr);
 				continuation = FALSE;
 				fieldname = TRUE;
 				valid = TRUE;
@@ -1006,6 +1027,9 @@ parser_step_headers (GMimeParser *parser)
 	if (priv->headerptr > priv->headerbuf)
 		header_parse (parser, &hend);
 	
+	if (priv->headers_end == -1)
+		priv->headers_end = parser_offset (priv, start);
+	
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
 	*priv->rawptr = '\0';
 	priv->inptr = inptr;
@@ -1014,6 +1038,9 @@ parser_step_headers (GMimeParser *parser)
 	
  next_message:
 	
+	if (priv->headers_end == -1)
+		priv->headers_end = parser_offset (priv, start);
+	
 	priv->state = GMIME_PARSER_STATE_COMPLETE;
 	*priv->rawptr = '\0';
 	priv->inptr = start;
@@ -1022,6 +1049,9 @@ parser_step_headers (GMimeParser *parser)
 	
  content_start:
 	
+	if (priv->headers_end == -1)
+		priv->headers_end = parser_offset (priv, start);
+	
 	priv->state = GMIME_PARSER_STATE_CONTENT;
 	*priv->rawptr = '\0';
 	priv->inptr = start;
@@ -1029,16 +1059,49 @@ parser_step_headers (GMimeParser *parser)
 	return 0;
 }
 
-static GMimeContentType *
+static void
+content_type_destroy (ContentType *content_type)
+{
+	g_free (content_type->subtype);
+	g_free (content_type->type);
+	g_free (content_type);
+}
+
+static gboolean
+content_type_is_type (ContentType *content_type, const char *type, const char *subtype)
+{
+	if (!strcmp (type, "*") || !g_ascii_strcasecmp (content_type->type, type)) {
+		if (!strcmp (subtype, "*")) {
+			/* special case */
+			return TRUE;
+		}
+		
+		if (!g_ascii_strcasecmp (content_type->subtype, subtype))
+			return TRUE;
+	}
+	
+	return FALSE;
+}
+
+static ContentType *
 parser_content_type (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	const char *content_type;
+	register const char *inptr;
+	ContentType *content_type;
+	const char *value;
 	
-	if ((content_type = header_raw_find (priv->headers, "Content-Type", NULL)))
-		return g_mime_content_type_new_from_string (content_type);
+	content_type = g_new (ContentType, 1);
 	
-	return NULL;
+	if (!(value = header_raw_find (priv->headers, "Content-Type", NULL)) ||
+	    !g_mime_parse_content_type (&value, &content_type->type, &content_type->subtype)) {
+		content_type->type = g_strdup ("text");
+		content_type->subtype = g_strdup ("plain");
+	}
+	
+	content_type->exists = value != NULL;
+	
+	return content_type;
 }
 
 static int
@@ -1062,7 +1125,7 @@ parser_skip_line (GMimeParser *parser)
 		
 		priv->inptr = inptr;
 		
-		if (parser_fill (parser) <= 0) {
+		if (parser_fill (parser, SCAN_HEAD) <= 0) {
 			inptr = priv->inptr;
 			rv = -1;
 			break;
@@ -1085,12 +1148,16 @@ parser_step (GMimeParser *parser)
 	case GMIME_PARSER_STATE_ERROR:
 		break;
 	case GMIME_PARSER_STATE_INIT:
+		parser->priv->headers_begin = -1;
+		parser->priv->headers_end = -1;
 		if (priv->scan_from)
 			priv->state = GMIME_PARSER_STATE_FROM;
 		else
 			priv->state = GMIME_PARSER_STATE_HEADERS;
 		break;
 	case GMIME_PARSER_STATE_FROM:
+		parser->priv->headers_begin = -1;
+		parser->priv->headers_end = -1;
 		parser_step_from (parser);
 		break;
 	case GMIME_PARSER_STATE_HEADERS:
@@ -1169,7 +1236,7 @@ check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
 		d(printf ("'%.*s' not a boundary\n", len, start));
 	}
 	
-	return 0;
+	return FOUND_NOTHING;
 }
 
 /* Optimization Notes:
@@ -1182,6 +1249,10 @@ check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
  * job correctly ;-)
  **/
 
+
+/* we add 2 for \r\n */
+#define MAX_BOUNDARY_LEN(bounds) (bounds ? bounds->boundarylenmax + 2 : 0)
+
 static int
 parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 {
@@ -1189,6 +1260,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 	register char *inptr;
 	char *start, *inend;
 	size_t nleft, len;
+	size_t atleast;
 	int found = 0;
 	
 	d(printf ("scan-content\n"));
@@ -1199,10 +1271,13 @@ parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 	
 	start = inptr = priv->inptr;
 	
+	/* figure out minimum amount of data we need */
+	atleast = MAX (SCAN_HEAD, MAX_BOUNDARY_LEN (priv->bounds));
+	
 	do {
 	refill:
 		nleft = priv->inend - inptr;
-		if (parser_fill (parser) <= 0) {
+		if (parser_fill (parser, atleast) <= 0) {
 			start = priv->inptr;
 			found = FOUND_EOS;
 			break;
@@ -1320,8 +1395,8 @@ static void
 parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *found)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	GMimeContentType *content_type;
 	struct _header_raw *header;
+	ContentType *content_type;
 	GMimeMessage *message;
 	GMimeObject *object;
 	
@@ -1343,14 +1418,13 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 		header = header->next;
 	}
 	
-	if (!(content_type = parser_content_type (parser)))
-		content_type = g_mime_content_type_new ("text", "plain");
-	
-	if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+	content_type = parser_content_type (parser);
+	if (content_type_is_type (content_type, "multipart", "*"))
 		object = parser_construct_multipart (parser, content_type, found);
 	else
 		object = parser_construct_leaf_part (parser, content_type, found);
 	
+	content_type_destroy (content_type);
 	message->mime_part = object;
 	
 	g_mime_message_part_set_message (mpart, message);
@@ -1358,7 +1432,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 }
 
 static GMimeObject *
-parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type, int *found)
+parser_construct_leaf_part (GMimeParser *parser, ContentType *content_type, int *found)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _header_raw *header;
@@ -1368,6 +1442,14 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
 	
+	if (!content_type->exists) {
+		GMimeContentType *mime_type;
+		
+		mime_type = g_mime_content_type_new ("text", "plain");
+		_g_mime_object_set_content_type (object, mime_type);
+		g_object_unref (mime_type);
+	}
+	
 	header = priv->headers;
 	while (header) {
 		g_mime_object_append_header (object, header->name, header->value);
@@ -1376,10 +1458,6 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
-	
 	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
@@ -1469,7 +1547,7 @@ static int
 parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	GMimeContentType *content_type;
+	ContentType *content_type;
 	GMimeObject *subpart;
 	int found;
 	
@@ -1492,15 +1570,14 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 			break;
 		}
 		
-		if (!(content_type = parser_content_type (parser)))
-			content_type = g_mime_content_type_new ("text", "plain");
-		
-		if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+		content_type = parser_content_type (parser);
+		if (content_type_is_type (content_type, "multipart", "*"))
 			subpart = parser_construct_multipart (parser, content_type, &found);
 		else
 			subpart = parser_construct_leaf_part (parser, content_type, &found);
 		
-		g_mime_multipart_add_part (multipart, subpart);
+		g_mime_multipart_add (multipart, subpart);
+		content_type_destroy (content_type);
 		g_object_unref (subpart);
 	} while (found == FOUND_BOUNDARY && found_immediate_boundary (priv, FALSE));
 	
@@ -1508,7 +1585,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 }
 
 static GMimeObject *
-parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type, int *found)
+parser_construct_multipart (GMimeParser *parser, ContentType *content_type, int *found)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _header_raw *header;
@@ -1528,10 +1605,6 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	if (object->content_type)
-		g_mime_content_type_destroy (object->content_type);
-	object->content_type = content_type;
-	
 	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
@@ -1545,7 +1618,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		}
 	}
 	
-	boundary = g_mime_content_type_get_parameter (content_type, "boundary");
+	boundary = g_mime_object_get_content_type_parameter (object, "boundary");
 	if (boundary) {
 		parser_push_boundary (parser, boundary);
 		
@@ -1575,7 +1648,7 @@ static GMimeObject *
 parser_construct_part (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	GMimeContentType *content_type;
+	ContentType *content_type;
 	GMimeObject *object;
 	int found;
 	
@@ -1585,21 +1658,21 @@ parser_construct_part (GMimeParser *parser)
 			return NULL;
 	}
 	
-	if (!(content_type = parser_content_type (parser)))
-		content_type = g_mime_content_type_new ("text", "plain");
-	
-	if (g_mime_content_type_is_type (content_type, "multipart", "*"))
+	content_type = parser_content_type (parser);
+	if (content_type_is_type (content_type, "multipart", "*"))
 		object = parser_construct_multipart (parser, content_type, &found);
 	else
 		object = parser_construct_leaf_part (parser, content_type, &found);
 	
+	content_type_destroy (content_type);
+	
 	return object;
 }
 
 
 /**
  * g_mime_parser_construct_part:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Constructs a MIME part from @parser.
  *
@@ -1619,8 +1692,8 @@ parser_construct_message (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	unsigned long content_length = ULONG_MAX;
-	GMimeContentType *content_type;
 	struct _header_raw *header;
+	ContentType *content_type;
 	GMimeMessage *message;
 	GMimeObject *object;
 	char *endptr;
@@ -1657,14 +1730,13 @@ parser_construct_message (GMimeParser *parser)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
 	
-	if (!(content_type = parser_content_type (parser)))
-		content_type = g_mime_content_type_new ("text", "plain");
-	
-	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*"))
+	content_type = parser_content_type (parser);
+	if (content_type_is_type (content_type, "multipart", "*"))
 		object = parser_construct_multipart (parser, content_type, &found);
 	else
 		object = parser_construct_leaf_part (parser, content_type, &found);
 	
+	content_type_destroy (content_type);
 	message->mime_part = object;
 	
 	if (priv->scan_from) {
@@ -1678,7 +1750,7 @@ parser_construct_message (GMimeParser *parser)
 
 /**
  * g_mime_parser_construct_message:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Constructs a MIME message from @parser.
  *
@@ -1695,7 +1767,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
 
 /**
  * g_mime_parser_get_from:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets the mbox-style From-line of the most recently parsed message
  * (gotten from g_mime_parser_construct_message()).
@@ -1723,13 +1795,13 @@ g_mime_parser_get_from (GMimeParser *parser)
 
 /**
  * g_mime_parser_get_from_offset:
- * @parser: MIME parser object
+ * @parser: a #GMimeParser context
  *
  * Gets the offset of the most recently parsed mbox-style From-line
  * (gotten from g_mime_parser_construct_message()).
  *
  * Returns: the offset of the most recently parsed mbox-style From-line
- * or -1 on error.
+ * or %-1 on error.
  **/
 gint64
 g_mime_parser_get_from_offset (GMimeParser *parser)
@@ -1744,3 +1816,41 @@ g_mime_parser_get_from_offset (GMimeParser *parser)
 	
 	return priv->from_offset;
 }
+
+
+/**
+ * g_mime_parser_get_headers_begin:
+ * @parser: a #GMimeParser context
+ *
+ * Gets the stream offset of the beginning of the headers of the most
+ * recently parsed message.
+ *
+ * Returns: the offset of the beginning of the headers of the most
+ * recently parsed message or %-1 on error.
+ **/
+gint64
+g_mime_parser_get_headers_begin (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
+	
+	return parser->priv->headers_begin;
+}
+
+
+/**
+ * g_mime_parser_get_headers_end:
+ * @parser: a #GMimeParser context
+ *
+ * Gets the stream offset of the end of the headers of the most
+ * recently parsed message.
+ *
+ * Returns: the offset of the end of the headers of the most recently
+ * parsed message or %-1 on error.
+ **/
+gint64
+g_mime_parser_get_headers_end (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
+	
+	return parser->priv->headers_end;
+}
