@@ -302,7 +302,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = FALSE;
 	fstream->fp = GMIME_STREAM_FILE (stream)->fp;
@@ -320,7 +320,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
  * Note: The created GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
- * Returns a stream using @fp.
+ * Returns: a stream using @fp.
  **/
 GMimeStream *
 g_mime_stream_file_new (FILE *fp)
@@ -331,7 +331,7 @@ g_mime_stream_file_new (FILE *fp)
 	if ((start = ftell (fp)) == -1)
 		start = 0;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, -1);
 	fstream->owner = TRUE;
 	fstream->fp = fp;
@@ -352,14 +352,14 @@ g_mime_stream_file_new (FILE *fp)
  * Note: The created GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
- * Returns a stream using @fp with bounds @start and @end.
+ * Returns: a stream using @fp with bounds @start and @end.
  **/
 GMimeStream *
 g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
-	fstream = g_object_new (GMIME_TYPE_STREAM_FILE, NULL);
+	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = TRUE;
 	fstream->fp = fp;
@@ -374,7 +374,7 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
  *
  * Gets whether or not @stream owns the backend FILE pointer.
  *
- * Returns %TRUE if @stream owns the backend FILE pointer or %FALSE
+ * Returns: %TRUE if @stream owns the backend FILE pointer or %FALSE
  * otherwise.
  **/
 gboolean
