@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -53,10 +53,10 @@ static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
-static off_t stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
-static off_t stream_tell (GMimeStream *stream);
+static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
+static gint64 stream_tell (GMimeStream *stream);
 static ssize_t stream_length (GMimeStream *stream);
-static GMimeStream *stream_substream (GMimeStream *stream, off_t start, off_t end);
+static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
 static GObjectClass *parent_class = NULL;
@@ -80,7 +80,8 @@ g_mime_stream_get_type (void)
 			(GInstanceInitFunc) g_mime_stream_init,
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GMimeStream", &info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "GMimeStream",
+					       &info, G_TYPE_FLAG_ABSTRACT);
 	}
 	
 	return type;
@@ -139,7 +140,7 @@ g_mime_stream_finalize (GObject *object)
  * Initializes a new stream with bounds @start and @end.
  **/
 void
-g_mime_stream_construct (GMimeStream *stream, off_t start, off_t end)
+g_mime_stream_construct (GMimeStream *stream, gint64 start, gint64 end)
 {
 	stream->position = start;
 	stream->bound_start = start;
@@ -163,7 +164,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
  *
  * Attempts to read up to @len bytes from @stream into @buf.
  *
- * Returns the number of bytes read or %-1 on fail.
+ * Returns: the number of bytes read or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_read (GMimeStream *stream, char *buf, size_t len)
@@ -194,7 +195,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
  *
  * Attempts to write up to @len bytes of @buf to @stream.
  *
- * Returns the number of bytes written or %-1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_write (GMimeStream *stream, const char *buf, size_t len)
@@ -223,7 +224,7 @@ stream_flush (GMimeStream *stream)
  *
  * Sync's the stream to disk.
  *
- * Returns %0 on success or %-1 on fail.
+ * Returns: %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_flush (GMimeStream *stream)
@@ -248,7 +249,7 @@ stream_close (GMimeStream *stream)
  *
  * Closes the stream.
  *
- * Returns %0 on success or %-1 on fail.
+ * Returns: %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_close (GMimeStream *stream)
@@ -273,7 +274,7 @@ stream_eos (GMimeStream *stream)
  *
  * Tests the end-of-stream indicator for @stream.
  *
- * Returns %TRUE on EOS or %FALSE otherwise.
+ * Returns: %TRUE on EOS or %FALSE otherwise.
  **/
 gboolean
 g_mime_stream_eos (GMimeStream *stream)
@@ -301,7 +302,7 @@ stream_reset (GMimeStream *stream)
  *
  * Resets the stream.
  *
- * Returns %0 on success or %-1 on fail.
+ * Returns: %0 on success or %-1 on fail.
  **/
 int
 g_mime_stream_reset (GMimeStream *stream)
@@ -317,8 +318,8 @@ g_mime_stream_reset (GMimeStream *stream)
 }
 
 
-static off_t
-stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
+static gint64
+stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	d(g_warning ("Invoked default stream_seek implementation."));
 	return -1;
@@ -344,10 +345,10 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
  *     #GMIME_STREAM_SEEK_END: Seek @offset bytes relative to the
  *     end of the stream (bound_end if non-negative).
  *
- * Returns the resultant position on success or %-1 on fail.
+ * Returns: the resultant position on success or %-1 on fail.
  **/
-off_t
-g_mime_stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
+gint64
+g_mime_stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
@@ -355,7 +356,7 @@ g_mime_stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 }
 
 
-static off_t
+static gint64
 stream_tell (GMimeStream *stream)
 {
 	d(g_warning ("Invoked default stream_tell implementation."));
@@ -369,9 +370,9 @@ stream_tell (GMimeStream *stream)
  *
  * Gets the current offset within the stream.
  *
- * Returns the current position within the stream or %-1 on fail.
+ * Returns: the current position within the stream or %-1 on fail.
  **/
-off_t
+gint64
 g_mime_stream_tell (GMimeStream *stream)
 {
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
@@ -394,7 +395,7 @@ stream_length (GMimeStream *stream)
  *
  * Gets the length of the stream.
  *
- * Returns the length of the stream or %-1 if unknown.
+ * Returns: the length of the stream or %-1 if unknown.
  **/
 ssize_t
 g_mime_stream_length (GMimeStream *stream)
@@ -406,7 +407,7 @@ g_mime_stream_length (GMimeStream *stream)
 
 
 static GMimeStream *
-stream_substream (GMimeStream *stream, off_t start, off_t end)
+stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	d(g_warning ("Invoked default stream_tell implementation."));
 	return NULL;
@@ -421,10 +422,10 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
  *
  * Creates a new substream of @stream with bounds @start and @end.
  *
- * Returns a substream of @stream with bounds @start and @end.
+ * Returns: a substream of @stream with bounds @start and @end.
  **/
 GMimeStream *
-g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
+g_mime_stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStream *sub;
 	
@@ -439,40 +440,6 @@ g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
 }
 
 
-/**
- * g_mime_stream_ref:
- * @stream: stream
- *
- * Ref's a stream.
- *
- * WARNING: This method is deprecated. Use g_object_ref() instead.
- **/
-void
-g_mime_stream_ref (GMimeStream *stream)
-{
-	g_return_if_fail (GMIME_IS_STREAM (stream));
-	
-	g_object_ref (stream);
-}
-
-
-/**
- * g_mime_stream_unref:
- * @stream: stream
- *
- * Unref's a stream.
- *
- * WARNING: This method is deprecated. Use g_object_unref() instead.
- **/
-void
-g_mime_stream_unref (GMimeStream *stream)
-{
-	g_return_if_fail (GMIME_IS_STREAM (stream));
-	
-	g_object_unref (stream);
-}
-
-
 /**
  * g_mime_stream_set_bounds:
  * @stream: stream
@@ -482,7 +449,7 @@ g_mime_stream_unref (GMimeStream *stream)
  * Set the bounds on a stream.
  **/
 void
-g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end)
+g_mime_stream_set_bounds (GMimeStream *stream, gint64 start, gint64 end)
 {
 	g_return_if_fail (GMIME_IS_STREAM (stream));
 	
@@ -499,19 +466,19 @@ g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end)
 /**
  * g_mime_stream_write_string:
  * @stream: stream
- * @string: string to write
+ * @str: string to write
  *
  * Writes @string to @stream.
  *
- * Returns the number of bytes written or %-1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
-g_mime_stream_write_string (GMimeStream *stream, const char *string)
+g_mime_stream_write_string (GMimeStream *stream, const char *str)
 {
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
-	g_return_val_if_fail (string != NULL, -1);
+	g_return_val_if_fail (str != NULL, -1);
 	
-	return g_mime_stream_write (stream, (char *) string, strlen (string));
+	return g_mime_stream_write (stream, (char *) str, strlen (str));
 }
 
 
@@ -523,7 +490,7 @@ g_mime_stream_write_string (GMimeStream *stream, const char *string)
  *
  * Write formatted output to a stream.
  *
- * Returns the number of bytes written or %-1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
@@ -556,7 +523,7 @@ g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
  *
  * Attempts to write stream @src to stream @dest.
  *
- * Returns the number of bytes written or %-1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
@@ -598,7 +565,7 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
  *
  * Writes at most @count blocks described by @vector to @stream.
  *
- * Returns the number of bytes written or %-1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_stream_writev (GMimeStream *stream, GMimeStreamIOVector *vector, size_t count)
